﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace KonkeDemo
{
    public class ControlChangesWithWindows
    {


        private float TempFormX;//当前窗体的宽度
        private float TempFormY;//当前窗体的高度


        // 将控件的宽，高，左边距，顶边距和字体大小暂存到tag属性中
        // <param name="cons">递归控件中的控件</param>
        //传入参数当前窗口
        private void setTag(Control cons)
        {
            foreach (Control con in cons.Controls)
            {
                con.Tag = con.Width + ":" + con.Height + ":" + con.Left + ":" + con.Top + ":" + con.Font.Size;
                if (con.Controls.Count > 0)
                {
                    setTag(con);
                }
            }
        }

        //根据窗体大小调整控件大小
        //传入参数，窗体宽度缩放比例，窗体高度缩放比例，当前窗口
        private void setControls(float newx, float newy, Control cons)
        {
            //遍历窗体中的控件，重新设置控件的值
            foreach (Control con in cons.Controls)
            {
                if (con != null)
                {
                    string[] mytag = con.Tag?.ToString().Split(new char[] { ':' });//获取控件的Tag属性值，并分割后存储字符串数组
                    if (mytag != null && mytag.Length >= 5)
                    {
                        float a = System.Convert.ToSingle(mytag[0]) * newx;//根据窗体缩放比例确定控件的值，宽度
                        con.Width = (int)a;//宽度
                        a = System.Convert.ToSingle(mytag[1]) * newy;//高度
                        con.Height = (int)(a);
                        a = System.Convert.ToSingle(mytag[2]) * newx;//左边距离
                        con.Left = (int)(a);
                        a = System.Convert.ToSingle(mytag[3]) * newy;//上边缘距离
                        con.Top = (int)(a);
                        Single currentSize = System.Convert.ToSingle(mytag[4]) * newy;//字体大小
                        con.Font = new Font(con.Font.Name, currentSize, con.Font.Style, con.Font.Unit);
                        if (con.Controls.Count > 0)
                        {
                            setControls(newx, newy, con);
                        }
                    }
                }
            }
        }


        //传入参数：当前窗口
        //用于窗口load事件调用
        public void MethodForLoad(Control cons)
        {
            TempFormX = cons.Width;//获取窗体的宽度
            TempFormY = cons.Height;//获取窗体的高度
            setTag(cons);
        }

        /// <summary>
        /// 窗口load事件调用，设置控件初始大小
        /// </summary>
        /// <param name="cons">当前窗口</param>
        /// <param name="ctlSizeReferent">大小变化的参考控件</param>
        public void MethodForLoad(Control cons, Control ctlSizeReferent)
        {
            TempFormX = ctlSizeReferent.Width;//获取窗体的宽度
            TempFormY = ctlSizeReferent.Height;//获取窗体的高度

            setTag(cons);
        }


        //引用SendMessage函数
        [DllImport("user32.dll", EntryPoint = "SendMessage")]
        private static extern int SendMessage(IntPtr hwnd, int wMsg, int wParam, IntPtr lParam);


        //传入参数:当前窗口
        //用于窗口Resize事件调用
        public void MethodForResize(Control cons)
        {
            float newx = (cons.Width) / TempFormX; //窗体宽度缩放比例
            float newy = (cons.Height) / TempFormY;//窗体高度缩放比例

            SendMessage(cons.Handle, 0xB, 0, IntPtr.Zero);  //解决控件较多时，改变大小闪一下问题

            setControls(newx, newy, cons);

            SendMessage(cons.Handle, 0XB, 1, IntPtr.Zero);
            cons.Invalidate(true);
        }

        /// <summary>
        /// 窗口Resize事件调用
        /// </summary>
        /// <param name="cons">当前窗口</param>
        /// <param name="ctlSizeReferent">大小变化的参考控件</param>
        public void MethodForResize(Control cons, Control ctlSizeReferent)
        {
            float newx = (ctlSizeReferent.Width) / TempFormX; //窗体宽度缩放比例
            float newy = (ctlSizeReferent.Height) / TempFormY;//窗体高度缩放比例

            SendMessage(cons.Handle, 0xB, 0, IntPtr.Zero);  //解决控件较多时，改变大小闪一下问题

            setControls(newx, newy, cons);

            SendMessage(cons.Handle, 0XB, 1, IntPtr.Zero);
            cons.Invalidate(true);
        }
    }
}
